package eu.e99.playerheads;

import com.google.gson.*;
import com.mojang.authlib.GameProfile;
import com.mojang.authlib.properties.Property;
import com.mojang.authlib.properties.PropertyMap;
import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import net.minecraft.component.DataComponentTypes;
import net.minecraft.component.type.ProfileComponent;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.item.ItemStack;
import net.minecraft.item.Items;
import net.minecraft.loot.condition.LootCondition;
import net.minecraft.loot.context.LootContext;
import net.minecraft.loot.function.ConditionalLootFunction;
import net.minecraft.loot.function.LootFunctionType;
import net.minecraft.util.context.ContextParameter;

import java.nio.charset.StandardCharsets;
import java.util.*;

public class CopyPlayerHeadLootFunction extends ConditionalLootFunction {

    public static final MapCodec<CopyPlayerHeadLootFunction> CODEC = RecordCodecBuilder.mapCodec(in -> addConditionsField(in)
            .and(LootContext.EntityTarget.CODEC
                    .fieldOf("entity")
                    .forGetter(func -> func.entity))
            .apply(in, CopyPlayerHeadLootFunction::new));

    private final LootContext.EntityTarget entity;

    public CopyPlayerHeadLootFunction(List<LootCondition> conditions, LootContext.EntityTarget entity) {
        super(conditions);
        this.entity = entity;
    }

    @Override
    public LootFunctionType<CopyPlayerHeadLootFunction> getType() {
        return PlayerHeads.COPY_PLAYER_HEAD;
    }

    @Override
    public Set<ContextParameter<?>> getAllowedParameters() {
        return Set.of(this.entity.getParameter());
    }

    @Override
    public ItemStack process(ItemStack stack, LootContext context) {
        if (!stack.isOf(Items.PLAYER_HEAD)) {
            return stack;
        }

        Object entity = context.get(this.entity.getParameter());
        if (!(entity instanceof PlayerEntity player)) {
            return stack;
        }

        String url = this.getSkinUrl(player);
        if (url == null) {
            return stack;
        }

        ProfileComponent component = this.profileFromSkinUrl(url);

        stack.set(DataComponentTypes.PROFILE, component);
        return stack;
    }

    private ProfileComponent profileFromSkinUrl(String url) {
        JsonObject skin = new JsonObject();
        skin.addProperty("url", url);

        JsonObject textures = new JsonObject();
        textures.add("SKIN", skin);

        JsonObject json = new JsonObject();
        json.add("textures", textures);

        String raw = json.toString();
        byte[] bytes = raw.getBytes(StandardCharsets.UTF_8);
        String value = Base64.getEncoder().encodeToString(bytes);

        Property property = new Property("textures", value);
        PropertyMap properties = new PropertyMap();
        properties.put("textures", property);

        return new ProfileComponent(
                Optional.empty(),
                Optional.empty(),
                properties
        );
    }

    private String getSkinUrl(PlayerEntity player) {
        GameProfile profile = player.getGameProfile();
        PropertyMap map = profile.getProperties();
        Collection<Property> properties = map.values();
        Base64.Decoder decoder = Base64.getDecoder();

        for (Property property : properties) {
            String name = property.name();
            if (!name.equals("textures")) {
                continue;
            }

            // TODO: Clean this up
            String value = property.value();
            byte[] decoded = decoder.decode(value);
            String raw = new String(decoded, StandardCharsets.UTF_8);
            JsonElement element = JsonParser.parseString(raw);

            JsonObject json = element.getAsJsonObject();
            JsonObject textures = json.getAsJsonObject("textures");
            JsonObject skin = textures.getAsJsonObject("SKIN");
            String url = skin.get("url").getAsString();

            return url;
        }

        return null;
    }
}
